	;001 EQUS 01.ASM - 18F452

;CONSTANTS
;VARIOUS
W                 EQU 0	;WREG is destination
F                 EQU 1	;register is destination
A                 EQU 0
ACCESS            EQU 0
BANKED            EQU 1
FSR0              EQU 0	;to be used with instruction LFSR
FSR1              EQU 1	;to be used with instruction LFSR
FSR2              EQU 2	;to be used with instruction LFSR
FAST              EQU 1   
;following CALL, (BSR, STATUS & W) are copied to the fast stack
;following RETURN, (BSR, STATUS & W) are retrieved from the fast stack
;following RETFIE, (BSR, STATUS & W) are retrieved from the fast stack
;(the interrupt has copied them automatically to the fast stack).

;MASKs for AND operation - bit =0 is masked off
MASK_HI_NIBBLE	  EQU B'00001111'	;masks higher nibble
MASK_LO_NIBBLE	  EQU B'11110000'	;masks lower nibble
MASK_765432		    EQU B'00000011'	;masks bits 7, 6, 5, 4, 3 & 2
MASK_43210		    EQU B'11100000'	;masks bits 4, 3, 2, 1 & 0
MASK_76543		    EQU B'00000111'	;masks bits 7, 6, 5, 4 & 3
MASK_76210        EQU B'00111000' ;masks bits 7, 6, 2, 1, & 0
MASK_7654 		    EQU B'00001111'	;masks bits 7, 6, 5 & 4
MASK_76   		    EQU B'00111111'	;masks bits 7 & 6
MASK_10   		    EQU B'11111100'	;masks bits 1 & 0
MASK_FOR_TEST     EQU B'00000000' ;to be defined according to byte to test
MASK_NONE         EQU B'11111111' ;masks no bits - just a placeholder.

COUNT_TBASE_SLOW  EQU 5   ;slow time base =COUNT_TBASE_SLOW * fast t. base
COUNT_DEB_KEYB    EQU 8   ;debouncing period =COUNT_DEB_KEYB * fast t. base
COUNT_DEB_KEYP    EQU 8   ;debouncing period =COUNT_DEB_KEYP * fast t. base
COUNT_DEB_ENC     EQU 5   ;debouncing period =COUNT_DEB_ENC * fast t. base
COUNT_WAIT_ENC    EQU 50  ;waiting period =COUNT_WAIT_ENC * fast t. base

P_M_MAX           EQU 24  ;No of different possible menues
AD                EQU 1
AT                EQU 0
;------------------------------------------------------------------------------

;To compile according to clock frequency, only ONE of these MUST be UNCOMMENTED
;The others MUST be ALL commented out.

;CLOCK            EQU 4000000     ;4 MHz   - Tcy = 1000 nsec
CLOCK            EQU 10000000    ;10 MHz  - Tcy =  400 nsec
;CLOCK            EQU 12000000    ;12 MHz  - Tcy =  333 nsec
;CLOCK            EQU 16000000    ;16 MHz  - Tcy =  250 nsec
;CLOCK            EQU 20000000    ;20 MHz  - Tcy =  200 nsec
;CLOCK            EQU 40000000    ;40 MHz  - Tcy =  100 nsec
;CLOCK            EQU 48000000    ;48 MHz  - Tcy =   83 nsec
;------------------------------------------------------------------------------

;RAM MEMORY MANAGEMENT
LOWRAM_B0       EQU	H'0000'+16	;1st RAM byte to clean - BANK0
HIGRAM_B0       EQU	H'00FF'	;last RAM byte to clean - BANK0

LOWRAM_B1       EQU	H'0100'	;1st RAM byte to clean - BANK1
HIGRAM_B1       EQU	H'01FF'	;last RAM byte to clean - BANK1

LOWRAM_B2       EQU	H'0200'	;1st RAM byte to clean - BANK2
HIGRAM_B2       EQU	H'02FF'	;last RAM byte to clean - BANK2

LOWRAM_B3       EQU	H'0300'	;1st RAM byte to clean - BANK3
HIGRAM_B3       EQU	H'03FF'	;last RAM byte to clean - BANK3

LOWRAM_B4       EQU	H'0400'	;1st RAM byte to clean - BANK4
HIGRAM_B4       EQU	H'04FF'	;last RAM byte to clean - BANK4

LOWRAM_B5       EQU	H'0500'	;1st RAM byte to clean - BANK5
HIGRAM_B5       EQU	H'05FF'	;last RAM byte to clean - BANK5

LENGTH_TO_CLB0	EQU	LOW (HIGRAM_B0 - LOWRAM_B0 + 1) ;Bytes to clean - BANK0
LENGTH_TO_CLB1	EQU	LOW (HIGRAM_B1 - LOWRAM_B1 + 1) ;Bytes to clean - BANK1
LENGTH_TO_CLB2	EQU	LOW (HIGRAM_B2 - LOWRAM_B2 + 1) ;Bytes to clean - BANK2
LENGTH_TO_CLB3	EQU	LOW (HIGRAM_B3 - LOWRAM_B3 + 1) ;Bytes to clean - BANK3
LENGTH_TO_CLB4	EQU	LOW (HIGRAM_B4 - LOWRAM_B4 + 1) ;Bytes to clean - BANK4
LENGTH_TO_CLB5	EQU	LOW (HIGRAM_B5 - LOWRAM_B5 + 1) ;Bytes to clean - BANK5

CLEAN_VAL       EQU	H'78'	;value to write when cleaning memory - DEBUG
;------------------------------------------------------------------------------

;I/O PORTS - ASSOCIATED REGISTERS

KEYB_LAT        EQU LATB
KEYB_PORT       EQU PORTB
KEYB_DIR        EQU TRISB

LCD_LAT         EQU LATD
LCD_PORT        EQU PORTD
LCD_DIR         EQU TRISD

MOUSE_LAT       EQU LATD
MOUSE_PORT      EQU PORTD
MOUSE_DIR       EQU TRISD

INTERF_LAT      EQU LATA
INTERF_PORT     EQU PORTA
INTERF_DIR      EQU TRISA
;------------------------------------------------------------------------------

;I/O PINS

;LCD
#DEFINE LCD_E         LCD_LAT,2     ;OUT LCD-enable control
#DEFINE LCD_RS        LCD_LAT,3     ;OUT LCD-register selection control

;INTERFACE
#DEFINE LED_RED       INTERF_LAT,0  ;OUT LED
#DEFINE LED_GREEN     INTERF_LAT,1  ;OUT LED
#DEFINE PIN_DEBUG     INTERF_LAT,2  ;OUT pin to detect activity
#DEFINE PIN_MOUSE_ON  INTERF_LAT,3  ;OUT pin to provide +Vcc to mouse

;MOUSE
#DEFINE PIN_CLK_IN    MOUSE_PORT,0  ;mouse's CLOCK (INPUT)
#DEFINE PIN_DATA_IN   MOUSE_PORT,1  ;mouses's DATA (INPUT)

#DEFINE PIN_CLK_OUT   MOUSE_LAT,0   ;mouse's CLOCK (OUTPUT)
#DEFINE PIN_DATA_OUT  MOUSE_LAT,1   ;mouses's DATA (OUTPUT)
;------------------------------------------------------------------------------

;GENERAL PURPOSE REGISTERS                ;ACCESS BANK LOW (H'000' to H'07F')

  CBLOCK H'000'
  
  ;All CNTR registers go here
  CNTR_GRAL           ;counter for general use
  CNTR_TBASE_SLOW     ;counter for slow time base
  
  CNTR_BITS_RCVD      ;holds counting of received bits
  CNTR_BITS_TO_XMT    ;holds counting of transmitted bits
  CNTR_PARITY         ;holds count of bits received as 1
	
  CNTR_OUTER          ;counter for online delay routines
  CNTR_MIDDLE         ;counter for online delay routines
  CNTR_INNER          ;counter for online delay routines

  CNTR_DEB_KEYB       ;counters used to implement
  CNTR_DEB_KEYP       ;non-online delays, based on
  CNTR_DEB_ENC

	CNTR_01             ;repetitive decrements implemented in the routine
  CNTR_02             ;DECR COUNTERS, called from inside MAIN_LOOP.
	CNTR_03             ;While CNTR_DEB_XXXX are always "fast", counters 01
  CNTR_04             ;to 05 can be "fast" or "slow" as flagged by the
  CNTR_05             ;respective flag CNTR_0X_FAST (1 for fast - 0 for slow).
  CNTR_DIG_BCD        ;points to digit in the BCD buffer
  
  CNTR_TESTS          ;codes sent to mouse to test WRAP mode
  
  ;All PNTR registers go here
  PNTR_MENU           ;number of menu to be shown on the LCD
  PNTR_DIG_BCD        ;employed in the routine CONV_BCD6_NOT_PACKED_BIN24

	
  ;All TEMP registers go here
  TEMP_W              ;context saving - save W here
  TEMP_STATUS         ;context saving - save STATUS here
  TEMP_BSR            ;context saving - save BSR here
  TEMP    	          ;file for general, BRIEF use
  TEMP_LCD            ;buffer for LCD data
  

  ;All FLAGS registers go here
  FLAGS0              ;various flags - general use
  FLAGS1              ;various flags - general use
  FLAGS2              ;various flags - general use
  FLAGS_KEYB          ;keyboard flags
  FLAGS_KEYP          ;keypad flags
  FLAGS_ENC           ;encoder flags  

  
  ;All BUFFER registers go here
  DIG_BCD_0           ;BCD buffer
  DIG_BCD_1           ;BCD buffer
  DIG_BCD_2           ;BCD buffer
  DIG_BCD_3           ;BCD buffer
  DIG_BCD_4           ;BCD buffer
  DIG_BCD_5           ;BCD buffer
  DIG_BCD_6           ;BCD buffer
  DIG_BCD_7           ;BCD buffer
  DIG_BCD_8           ;BCD buffer
  DIG_BCD_IN          ;holds the selected digit value
  
  DIG_ASCII_L         ;holds lower byte in ASCII of an 8-bits value
  DIG_ASCII_H         ;holds higher byte in ASCII of an 8-bits value  
  DIG_ASCII_0         ;ASCII buffer
  DIG_ASCII_1         ;ASCII buffer
  DIG_ASCII_2         ;ASCII buffer
  DIG_ASCII_3         ;ASCII buffer
  DIG_ASCII_4         ;ASCII buffer
  DIG_ASCII_5         ;ASCII buffer
  DIG_ASCII_6         ;ASCII buffer
  DIG_ASCII_7         ;ASCII buffer
  DIG_ASCII_8         ;ASCII buffer
  DIG_ASCII_9         ;ASCII buffer
  DIG_ASCII_10        ;ASCII buffer      
  DIG_ASCII_11        ;ASCII buffer
  
  BUFF_DISP_0         ;display buffer
  BUFF_DISP_1         ;display buffer 
  BUFF_DISP_2         ;display buffer
  BUFF_DISP_3         ;display buffer
  BUFF_DISP_4         ;display buffer
  BUFF_DISP_5         ;display buffer
  BUFF_DISP_6         ;display buffer
  BUFF_DISP_7         ;display buffer
  BUFF_DISP_8         ;display buffer
  BUFF_DISP_9         ;display buffer
  BUFF_DISP_10        ;display buffer
  BUFF_DISP_11        ;display buffer
  
  BUFF_BITS_RCVD      ;used to input bits received from the mouse
  BUFF_BYTE_TO_XMT    ;used to hold the bits to transmit to mouse
  BUFF_READING        ;used to input bits read from mouse
  
  ;All GENERAL USE registers go 
  KEYB_COLS           ;active keyboard cols (7:4) recorded here
  KEYB_ROWS           ;active keyboard rows (3:0) recorded here
  KEYB_SCAN           ;holds the first keyb-matrix scan
  KEYP_READ           ;holds the first keyp reading
  MASK_TEST           ;register holding the mask for testing a byte's bits

	
	;All registers for maths go here
  NROA_U              ;nro A
  NROA_H              ;used in
  NROA_L              ;math routines
  
  NROB_U              ;nro B
  NROB_H              ;used in
  NROB_L              ;math routines
  
  NROBIN_U            ;24-bits binary numbers
  NROBIN_H            ;16-bits used
  NROBIN_L            ;08-bits in math routines
  
  RES_3               ;result
  RES_2               ;from the
  RES_1               ;multiplication
  RES_0               ;routine
  
  DEND_5              ;dividend
  DEND_4              ;for
  DEND_3              ;the
  DEND_2              ;division
  DEND_1              ;routines
  DEND_0              ;48 or 32-bits
  
  DSOR_U              ;divisor for the
  DSOR_H              ;division routines
  DSOR_L              ;24 or 16-bits
	ENDC
	
;following values defined here, AFTER the DIG_BCD_X definitions above!!!

DIG_BCD_87  EQU DIG_BCD_7
DIG_BCD_65  EQU DIG_BCD_5
DIG_BCD_43  EQU DIG_BCD_3
DIG_BCD_21  EQU DIG_BCD_1
;------------------------------------------------------------------------------

;BITS

;FLAGS
#DEFINE TBASE_FAST_ELAP FLAGS0,0  ;=1 a period of fast time base, has elapsed
#DEFINE TBASE_SLOW_ELAP FLAGS0,1  ;=1 a period of slow time base, has elapsed

#DEFINE	CNTR_01_FAST    FLAGS1,1  ;=1 uses fast timebase/ =0 uses slow timebase
#DEFINE	CNTR_02_FAST    FLAGS1,2  ;=1 uses fast timebase/ =0 uses slow timebase
#DEFINE	CNTR_03_FAST    FLAGS1,3  ;=1 uses fast timebase/ =0 uses slow timebase
#DEFINE	CNTR_04_FAST    FLAGS1,4  ;=1 uses fast timebase/ =0 uses slow timebase
#DEFINE	CNTR_05_FAST    FLAGS1,5  ;=1 uses fast timebase/ =0 uses slow timebase

#DEFINE	BIT_FOR_ODD     FLAGS2,0  ;=1 value to write as parity bit

#DEFINE	KEY_ACTIVE      FLAGS_KEYB,7    ;=1 a key is "active" (still pressed)
#DEFINE	SCAN_1ST_DONE   FLAGS_KEYB,6    ;=1 first matrix scan was completed
#DEFINE ROW_ACTIVE      KEYB_ROWS,7     ;=1 the row just read is active.

#DEFINE BTN_ACTIVE      FLAGS_KEYP,7    ;=1 a button is "active" (still pressed)
#DEFINE	READ_1ST_DONE   FLAGS_KEYP,6    ;=1 first keypad reading was completed
