	;004 MACROS 01.ASM	18F family

;------------------------------------------------------------------------------
;REG:    REGISTER
;VAL:    VALUE
;------------------------------------------------------------------------------

LOADREG   MACRO REG,VAL
    	    MOVLW VAL
    	    MOVWF REG
    	    ENDM
;------------------------------------------------------------------------------

LOAD2REGS	MACRO REG1,REG2,VAL
    	    MOVLW VAL
    	    MOVWF REG1
    	    MOVWF REG2
    	    ENDM
;------------------------------------------------------------------------------

LOAD3REGS	MACRO REG1,REG2,REG3,VAL
    	    MOVLW VAL
    	    MOVWF REG1
    	    MOVWF REG2
    	    MOVWF REG3
    	    ENDM
;------------------------------------------------------------------------------

SWAPREGS	MACRO REG1,REG2
    	    MOVF REG1,W
    	    XORWF REG2,W
    	    XORWF REG2,F
    	    XORWF REG1,F
    	    ENDM
;------------------------------------------------------------------------------

SKIP_R_DIF_ZERO	MACRO REG           ;skip line after macro if REG is not =0
                MOVF REG,F          ;R copies to itself
                BTFSC STATUS,Z      ;if Z=0 REG !=0; skip next line!
                ENDM

SKIP_R_EQU_V	  MACRO REG,VAL       ;skip line after macro if REG =VAL
                MOVLW VAL           ;W =VAL against which we compare REG
                CPFSEQ REG          ;if REG=VAL, skip next line!
                ENDM

SKIP_R_HIG_V	  MACRO REG,VAL       ;skip line after macro if REG >VAL
                MOVLW VAL           ;W =VAL against which we compare REG
                CPFSGT REG          ;if REG>VAL, skip next line!
                ENDM

SKIP_R_LOW_V	  MACRO REG,VAL       ;skip line after macro if REG <VAL
                MOVLW VAL           ;W =VAL against which we compare REG
                CPFSLT REG          ;if REG<VAL, skip next line!
                ENDM

SKIP_R_DIF_V	  MACRO REG,VAL       ;skip line after macro if REG !=VAL
                MOVLW VAL           ;W =VAL which substracted from
                SUBWF REG,W         ;REG (but not changing it)
                BTFSC STATUS,Z      ;if Z=0 REG !=VAL; skip next line!
                ENDM

SKIP_R_DIF_255  MACRO REG           ;skip line after macro if REG !=255
                INCF REG,W          ;if REG=255, it would wrap over to 0
                BTFSC STATUS,Z      ;if Z=0, REG !=0; skip next line!
                ENDM
;------------------------------------------------------------------------------

SKIP_R_IN_RANGE_L_H	MACRO REG,LO_LIM,HI_LIM
        	        MOVF REG,W          ;W =REG
        	        ADDLW 255 - HI_LIM  ;W =REG - HI_LIM
        	        ADDLW HI_LIM-LO_LIM+1
        	
        	        ;W =REG - HI_LIM + HI_LIM-LO_LIM + 1
        	        BTFSS STATUS,C
        	        ENDM

SKIP_R_IN_RANGE_0_H	MACRO REG,HI_LIM  ;LO_LIM always =0
        	        MOVF REG,W          ;W =REG
        	        SUBLW HI_LIM
        	        BTFSS STATUS,C
        	        ENDM
;------------------------------------------------------------------------------

SKIP_V_HIG_R	      MACRO VAL,REG       ;skip line after macro if VAL>REG
                    MOVLW VAL           ;W =VAL against which we compare REG
                    CPFSLT REG          ;if REG<VAL (VAL>REG), skip next line!
                    ENDM

SKIP_V_LOW_R        MACRO VAL,REG       ;skip line after macro if VAL<REG
                    MOVLW VAL    	      ;W =VAL against which we compare REG
                    CPFSGT REG          ;if REG>VAL (VAL<REG), skip next line!
                    ENDM
;------------------------------------------------------------------------------

SKIP_R1_HIG_R2	    MACRO REG1,REG2
                    MOVF REG2,W    	;W =REG2
                    CPFSGT REG1    	;if REG1 >W, skip next line!
                    ENDM

SKIP_R1_EQU_R2	    MACRO REG1,REG2
                    MOVF REG2,W    	;W =REG2	
                    CPFSEQ REG1    	;if REG1 =W, skip next line!
                    ENDM

SKIP_R1_LOW_R2	    MACRO REG1,REG2
                    MOVF REG2,W    	;W =REG2
                    CPFSLT REG1    	;if REG1 <W, skip next line!
                    ENDM

SKIP_R1_DIF_R2	    MACRO REG1,REG2
                    MOVF REG2,W    	;W =REG2, substracted from
                    
                    SUBWF REG1,W    ;REG1 (but not changing it)
                    BTFSC STATUS,Z  ;Z=0? R1 !=R2 - skip line!
                    ENDM
;------------------------------------------------------------------------------

LOADWORD            MACRO WORD_H,WORD_L,VAL
                    MOVLW HIGH VAL  ;higher part of VAL loaded in the
                    MOVWF WORD_H    ;higher part of WORD
                    MOVLW LOW VAL   ;lower part of VAL loaded in the
                    MOVWF WORD_L    ;lower part of WORD
                    ENDM
;------------------------------------------------------------------------------

SKIP_WORD_ZERO	    MACRO WORD_H,WORD_L
                    MOVF WORD_H,W   ;W =WORD_H
                    IORWF WORD_L,W  ;W =W.OR.WORD_L will be zero if both =0
                    BTFSS STATUS,Z  ;if Z=1 (both=0) - skip next line!
                    ENDM
        	
SKIP_WORD_65535     MACRO WORD_H,WORD_L
                    MOVF WORD_H,W   ;W =WORD_H
                    ANDWF WORD_L,W  ;W.AND.WORD_L =255 if WORD_H and WORD_L =255
                    XORLW 255    	;W.XOR.255 =0, only if W was =255
                    BTFSS STATUS,Z  ;if Z =1, W =255 - skip next line!
                    ENDM
;------------------------------------------------------------------------------

SKIP_WORD_HIG_V     MACRO WORD_H,WORD_L,VAL
                    MOVLW HIGH VAL  ;W =VAL_H - substracts value from
                    SUBWF WORD_H,W  ;WORD_H (but not changing it)

                    BTFSS STATUS,C  ;if C =1, WORD_H =>VAL_H        	
                    BRA $+16    	  ;C =0, W_H <V, go to the line!
        
                    BTFSS STATUS,Z  ;if Z=1 WORD_H =VAL_H, check WORD_L
                    BRA $+14    	  ;Z =0, WORD_H>VAL_H; skip the line!
                  
                    MOVLW LOW VAL   ;W =VAL_L - substracts value from
                    SUBWF WORD_L,W  ;WORD_L (but not changing it)
        
                    BTFSS STATUS,C  ;if C =1, WORD_L=>VAL_L - Check Z
                    BRA $+4         ;C =0, W<V - go to the line!
        
                    BTFSC STATUS,Z  ;if Z =0, WORD_L>VAL_L; skip line!
                    ENDM

SKIP_WORD_EQU_V	    MACRO WORD_H,WORD_L,VAL
                    MOVF WORD_H,W   ;W =WORD_H
                    XORLW HIGH VAL  ;W.XOR.VAL_H - if equal, Z=1
                    BTFSS STATUS,Z  ;if Z=1; equal - check WORD_L!
                    BRA $+8         ;Z =0; not equal - don't skip!
        
                    MOVF WORD_L,W   ;W =WORD_L
                    XORLW LOW VAL   ;W.XOR.VAL_L - if both equal, Z=1
                    BTFSS STATUS,Z  ;if Z=1, WORD_L=VAL_L - skip line!
                    ENDM

SKIP_WORD_DIF_V	    MACRO WORD_H,WORD_L,VAL
                    MOVLW HIGH VAL  ;W =VAL_H - substracts value from
                    SUBWF WORD_H,W  ;WORD_H (but not changing it)
                    BTFSS STATUS,Z  ;if Z =1, W_H=V_H, check WORD_L
                    BRA $+10    	  ;Z =0; not equal - skip the line!
        
                    MOVLW LOW VAL   ;W =VAL_L - substracts value from
                    SUBWF WORD_L,W  ;WORD_L (but not changing it)
                    BTFSC STATUS,Z  ;if Z=0, F !=V - skip next line!
                    ENDM

SKIP_WORD_LOW_V	    MACRO WORD_H,WORD_L,VAL
                    MOVLW HIGH VAL  ;W =VAL_H - substracts value from
                    SUBWF WORD_H,W  ;WORD_H (but not changing it)
                    BTFSS STATUS,C  ;if C =1, WORD_H=>VAL_H - check Z
                    BRA $+14    	  ;C =0, W_H<V_H - skip the line!
        
                    BTFSS STATUS,Z  ;if Z =1 W_H=V_H - Check WORD_L
                    BRA $+8         ;Z =0, W_H>V_H - go to the line!
        
                    MOVLW LOW VAL   ;W =VAL_L - substracts value from
                    SUBWF WORD_L,W  ;WORD_L (but not changing it)
                    BTFSC STATUS,C  ;if C=0, W_L<V_L - skip next line!
                    ENDM
;------------------------------------------------------------------------------

SKIP_W1_EQU_W2	    MACRO WORD1_H,WORD1_L,WORD2_H,WORD2_L
                    MOVF WORD1_H,W  ;W =WORD1_H
                    XORWF WORD2_H,W ;W.XOR.WORD2_H - if both equal, Z=1
                    BTFSS STATUS,Z  ;if Z=1, W1_H =W2_H; check WORD1_L!
                    BRA $+8         ;Z=0 - not equal - go to line!
                    
                    MOVF WORD1_L,W  ;W=WORD1_L
                    XORWF WORD2_L,W ;W.XOR.WORD2_L - if both equal, Z=1        
                    BTFSS STATUS,Z  ;if Z=1, W1_L=W2_L; skip next line!
                    ENDM

SKIP_W1_DIF_W2	    MACRO WORD1_H,WORD1_L,WORD2_H,WORD2_L
                    MOVF WORD1_H,W  ;W =WORD1_H
                    XORWF WORD2_H,W ;W.XOR.WORD2_H - if both diff, Z=0
                    BTFSS STATUS,Z  ;if Z=1, W1_H=W2_H - check WORD2_H
                    BRA $+10    	  ;Z =0, W1_H !=W2_H; skip the line!
            
                    MOVF WORD1_L,W  ;W =WORD1_L
                    XORWF WORD2_L,W ;W.XOR.WORD2_L - if both diff, Z=0        
                    BTFSC STATUS,Z  ;if Z=0, W1_L !=W2_L; skip line!
                    ENDM

SKIP_W1_HIG_W2      MACRO WORD1_H,WORD1_L,WORD2_H,WORD2_L
                    MOVF WORD2_H,W  ;W =WORD2_H - substracted from
                    SUBWF WORD1_H,W ;WORD1_H (but not changing it)
                    BTFSS STATUS,C  ;if C =1, WORD1_H =>WORD2_H - check Z
                    BRA $+16    	  ;C =0 W1_H<W2_H - don't skip the line!
            
                    BTFSS STATUS,Z  ;if Z=1 WORD1_H=WORD2_H; check WORD1_L
                    BRA $+14    	  ;Z =0, W1_H>W2_H - skip the line!
            
                    MOVF WORD2_L,W  ;W =WORD2_L - substracted from
                    SUBWF WORD1_L,W ;WORD1_L (but not changing it)
                    BTFSS STATUS,C  ;if C =1, WORD1_L =>WORD2_L - check Z
                    BRA $+4         ;C=0 WORD1_L <WORD2_L;go to the line!
            
                    BTFSC STATUS,Z  ;if Z=0, W1_L >W2_L - skip next line!
                    ENDM

SKIP_W1_LOW_W2      MACRO WORD1_H,WORD1_L,WORD2_H,WORD2_L
                    MOVF WORD2_H,W  ;W =WORD2_H - substracted from
                    SUBWF WORD1_H,W ;WORD1_H (but not changing it)
                    BTFSS STATUS,C	;if C =1, WORD1_H=>WORD2_H - check Z
                    BRA $+14    	  ;C =0, W1_H <W2_H - skip the line!
            
                    BTFSS STATUS,Z  ;if Z =1 W1_H =W2_H - Check WORD1_L
                    BRA $+8         ;Z =0, W1_H >VL_H - don't skip!
                    
                    MOVF WORD2_L,W  ;W =WORD2_L - substracted from
                    SUBWF WORD1_L,W ;WORD1_L (but not changing it)
                    BTFSC STATUS,C  ;if C =0, W1_L<W2_L; skip next line!
                    ENDM
;--------------------------------------------------------------------------------------------

INCR_WORD           MACRO WORD_H,WORD_L 
                    INCF WORD_L
                    BTFSC STATUS,C
                    INCF WORD_H
                    ENDM

DECR_WORD           MACRO WORD_H,WORD_L 
                    DECF WORD_L
                    BTFSS STATUS,C
                    DECF WORD_H
                    ENDM
                    






