    ;019 INIT INTERRUPTS.ASM

;------------------------------------------------------------------------------

;1 - Setting the peripherics source of interrupt.

;A) TMR2 used to get the 5 ms time base. TMR2 resets to 0 in the next cycle
;after matching PR2, repeating the cycle indefinitely. After every match, an
;interrupt is requested. The respective ISR sets a flag, used by the
;DECR_COUNTERS routine, called continuously from main loop.

;TMR2 is fed through its prescaler with Tcy =1 us. To get 5 ms we need:
;5 ms / 1 us =5.000 counts; that is: [4 * 250 * 5] =5.000. To get that, we set:
;prescaler 1:4  -  PR2 =250  -  postscaler  1:5

    LOADREG T2CON,B'00100101'      ;set the T2CON controlling register
                  ; 0.......        b7 =0  XX
                  ; .0......        b6 =0  } 0100
                  ; ..1.....        b5 =1  } setting
                  ; ...0....        b4 =0  } for a 1:5
                  ; ....0...        b3 =0  } postscaler
                  ; .....1..        b2 =1  enable TMR2
                  ; ......0.        b1 =0  } 01 - setting for a
                  ; .......1        b0 =1  } 1:4 prescaler

    LOADREG PR2,250
;------------------------------------------------------------------------------

;B) TMR3 - timer mode - internal clock (Fosc/4) - used to materialize a timeout
;of XXX usec if the software does not read the start of a character (from the
;device) in the expected time (?? usec app.), avoiding to get stuck trying to
;read a character that won't come. This way it can read more than one character
;when their number is unknown beforehand.

;No ISR is triggered. Instead, while checking the start of a new character, the
;overflow flag is also polled. If the flag is found set any further reading is
;terminated and the routine exited.

;TIMER3 loaded with, say 65260 (H'FEEC'), has to count 275 ticks to overflow
;after 65535 (65260 + 275 =65535).

;T internal clock = (1/10E6) * 4 = 100nsec * 4 =400 nsec.
;Total time to overflow: 275 * 400 nsec =110 usec.

;In actual practice, it seems that every chip has a different delay between
;every character after the first. We have seen delays ranging from 81 usec to
;362 usec. This last is used in this version.

;A word of WARNING: try to adjust the timeout for few usec more than the delay
;measured for any particular chip. One for all, seems not to work. At least that
;is my experience.

;Prior reading an expected character:
;TIMER3 is reloaded with H'FEEC'
;TMR3IF is reset
;TMR3ON bit is set to start the timer.

;Register T3CON stays as after RESET:

;                 B'00000000'      ;settings of TMR3 controlling register
                  ; 0.......        b7 =0  writing done in two 8-bit operations
                  ; .0......        b6 =0  XXX
                  ; ..0.....        b5 =0  } prescale
                  ; ...0....        b4 =0  } value 1:1
                  ; ....0...        b3 =0  XXX
                  ; .....0..        b2 =0  XXX
                  ; ......0.        b1 =0  internal clock (Fosc/4)
                  ; .......0        b0 =0  timer is stopped
;------------------------------------------------------------------------------

;2 - Setting the priorities of interrupts set above.

    BSF RCON,IPEN           ;=1 enable interrupts priority scheme
    BSF IPR1,TMR2IP         ;=1 TMR2 interrupt has high priority
    
;if NO PRIORITIES, we keep clear bit RCON,IPEN (priority scheme disabled)
;------------------------------------------------------------------------------

;3 - Enable individually the sources of interrupt above.

    BSF PIE1,TMR2IE         ;=1 TMR2 match interrupt enabled
;------------------------------------------------------------------------------

;4 - Clear the flags of the interrupts above.

    BCF PIR1,TMR2IF         ;=0 clear TMR2 match interrupt flag
;------------------------------------------------------------------------------

;5 - Enabling interrupts according to priority.

    BSF INTCON,GIEH         ;=1 enables all high priority interrupts
;------------------------------------------------------------------------------
