    ;ROUT DECR COUNTERS.ASM

DECR_COUNTERS

;Use of counters for delays based on a suggestion from Stewart in the Microchip
;forum.

;Except when starting the LCD module, most of the delays (if not all) used
;in the different tasks, progress "in parallel" with the main loop code.

;A specific delay starts when its own counter is loaded with a >0 value and is
;materialized in the time it takes its repetitive "decrement until =0".

;Decrement done in this routine which is called from inside of MAIN_LOOP.

;The counter for each delay, is defined beforehand.

;There is the ISR in charge (triggered by the overflow of Timer2), setting the
;pace, materializing a time base, ticking every 5 msec, setting, every time,
;a "time base elapsed" flag.

;In fact, the ISR, sets the pace, materialized in two time bases, used to
;decrement the "fast" and "slow" counters.

;FAST counters are decremented when the fast timebase flag is detected set.
;SLOW counters are decremented when the slow timebase flag is detected set.

;The routine below, when that flag is set, decrements all non =0 counters.
;When a counter reaches =0 it stays like that forever unless reloaded with a
;new value (>0) somewhere inside the code.
;------------------------------------------------------------------------------

    BTFSS TBASE_FAST_ELAP   ;if new fast time base period elapsed...
    RETURN                  ;no new fast, thus no new slow either - leave!
    
DEBN_KEYB                   ;informative label
    TSTFSZ CNTR_DEB_KEYB    ;...if counter = 0, don't mess with it
    DECF CNTR_DEB_KEYB,F    ;not =0 - decrement counter and move to next 

FAST_1                      ;informative label
    BTFSS CNTR_01_FAST      ;is this a fast counter?
    BRA FAST_2              ;no - check the next one
    TSTFSZ CNTR_01          ;it is fast - if =0, skipt it!
    DECF CNTR_01,F          ;not =0 - decrement counter and move to next
    
FAST_2                      ;informative label
    BTFSS CNTR_02_FAST      ;is this a fast counter?
    BRA FAST_3              ;no - check the next one
    TSTFSZ CNTR_02          ;it is fast - if =0, skipt it!
    DECF CNTR_02,F          ;not =0 - decrement counter and move to next    
    
FAST_3                      ;informative label
    BTFSS CNTR_03_FAST      ;is this a fast counter?
    BRA FAST_4              ;no - check the next one
    TSTFSZ CNTR_03          ;it is fast - if =0, skipt it!
    DECF CNTR_03,F          ;not =0 - decrement counter and move to next    
    
FAST_4                      ;informative label
    BTFSS CNTR_04_FAST      ;is this a fast counter?
    BRA FAST_5              ;no - check the next one
    TSTFSZ CNTR_04          ;it is fast - if =0, skipt it!
    DECF CNTR_04,F          ;not =0 - decrement counter and move to next
    
FAST_5
    BTFSS CNTR_05_FAST       ;is this a fast counter?
    BRA FAST_CNTRS_DONE      ;no - done with fast counters
    TSTFSZ CNTR_05           ;it is fast - if =0, skipt it!   
    DECF CNTR_05,F           ;not =0 - decrement counter and move

FAST_CNTRS_DONE
    BCF TBASE_FAST_ELAP     ;prepare for next fast period! - reset flag
;------------------------------------------------------------------------------

DECR_SLOW_CNTRS
    BTFSS TBASE_SLOW_ELAP   ;if new slow time base period elapsed...
    RETURN                  ;no new slow time base - leave!

SLOW_1                      ;informative label
    BTFSC CNTR_01_FAST      ;is this a slow counter?
    BRA SLOW_2              ;no - check the next one
    TSTFSZ CNTR_01          ;it is slow - if =0, skipt it!
    DECF CNTR_01,F          ;not =0 - decrement counter and move to next
    
SLOW_2    
    BTFSC CNTR_02_FAST      ;is this a slow counter?
    BRA SLOW_3              ;no - check the next one
    TSTFSZ CNTR_02          ;it is slow - if =0, skipt it!
    DECF CNTR_02,F          ;not =0 - decrement counter and move to next
    
SLOW_3    
    BTFSC CNTR_03_FAST      ;is this a slow counter?
    BRA SLOW_4              ;no - check the next one
    TSTFSZ CNTR_03          ;it is slow - if =0, skipt it!
    DECF CNTR_03,F          ;not =0 - decrement counter and move to next

SLOW_4
    BTFSC CNTR_04_FAST      ;is this a slow counter?
    BRA SLOW_5              ;no - check the next one
    TSTFSZ CNTR_04          ;it is slow - if =0, skipt it!
    DECF CNTR_04,F          ;not =0 - decrement counter and move to next    
    
SLOW_5
    BTFSC CNTR_05_FAST      ;is this a slow counter?
    BRA SLOW_CNTRS_DONE     ;no - done with slow counters
    TSTFSZ CNTR_05          ;it is slow - if =0, skipt it  
    DECF CNTR_05,F          ;not =0 - decrement counter and move

SLOW_CNTRS_DONE
    BCF TBASE_SLOW_ELAP     ;prepare for next slow period! - reset flag
    RETURN                  ;fast and slow decrements done - leave!
