	;ROUT LCD.ASM

LCD_LINE

;To call this routine:
;TEMP_LCD loaded with desired cursor address (hex value) in the LCD display
;TBLPTRU:TBLPTRH:TBLPTRL with the address bytes where the text line starts

	CALL LCD_CRSR_LOCATE		        ;cursor position in TEMP_LCD passed to LCD
                                  ;now TEMP_LCD is used for char management	
LCD_LINE_LOOP
	TBLRD*+						              ;read the byte & incr TBLPTR - byte in TABLAT
	TSTFSZ TABLAT				            ;if TABLAT =0, we have reached...
	BRA LCD_NEXT_CHAR			          ;TABLAT >0 - process next char
	RETURN						              ;...the end of table - leave!	
	
LCD_NEXT_CHAR
	MOVFF TABLAT,TEMP_LCD		        ;ASCII code from TABLAT to TEMP_LCD
	CALL LCD_WRITE_DATA			        ;data in TEMP_LCD will be passed to LCD
	BRA LCD_LINE_LOOP			          ;let's go for next char in text table
;------------------------------------------------------------------------------

LCD_WRITE_INST
	CALL    DELAY_005_MILI			    ;delay replaces the BUSY flag check
	MOVF    TEMP_LCD,W			        ;copy instruction from TEMP_LCD into W
	ANDLW   MASK_LO_NIBBLE		      ;isolate upper nibble
	MOVWF   LCD_LAT     			      ;(b7:b4) nibble in the data bus
	WR_INST_TO_LCD				          ;tell the display we are writing instruction
	CALL PULSE_LCD_E			          ;activate writing of 1st part
	
	SWAPF   TEMP_LCD,W			        ;swap nibbles and copy to W
	ANDLW   MASK_LO_NIBBLE		      ;isolate new upper nibble
	MOVWF   LCD_LAT     			      ;(b7:b4) nibble in the data bus
	WR_INST_TO_LCD				          ;tell the display we are writing instruction	
	CALL PULSE_LCD_E			          ;activate writing of 2nd part
	RETURN
;------------------------------------------------------------------------------

LCD_WRITE_DATA
	CALL    DELAY_005_MILI			    ;delay replaces the BUSY flag check
	MOVF    TEMP_LCD,W			        ;copy data from TEMP_LCD into W
	ANDLW   MASK_LO_NIBBLE		      ;isolate upper nibble
	MOVWF   LCD_LAT     			      ;(b7:b4) nibble in the data bus
	WR_DATA_TO_LCD				          ;tell the display we are writing data
	CALL PULSE_LCD_E			          ;activate writing of 1st part
	
	SWAPF   TEMP_LCD,W			        ;swap nibbles and copy to W
	ANDLW   MASK_LO_NIBBLE		      ;isolate new upper nibble
	MOVWF   LCD_LAT      		        ;(b7:b4) nibble in the data bus
	WR_DATA_TO_LCD				          ;tell the display we are writing data
	CALL PULSE_LCD_E			          ;activate writing of 2nd part
	RETURN
;------------------------------------------------------------------------------

PULSE_LCD_E
	BSF LCD_E					              ;sets LCD's input E
	NOP                             ;just in case
	NOP                             ;just in case
	BCF LCD_E					              ;resets LCD's input E
	RETURN
;------------------------------------------------------------------------------

LCD_CRSR_HOME

	LOADREG TEMP_LCD,CRSR_HOME	    ;CRSR position: first col of first line
	CALL LCD_WRITE_INST		
	RETURN
;------------------------------------------------------------------------------

LCD_CRSR_LOCATE

;To call this routine:
;TEMP_LCD loaded with desired cursor address (hex value) in the LCD display

	BSF TEMP_LCD,7
;b7 set converts cursor position into an instruction (Winstar manual).
	CALL LCD_WRITE_INST			        ;cursor position in TEMP_LCD passed to LCD
	RETURN
;------------------------------------------------------------------------------

LCD_DISP_OFF

	LOADREG TEMP_LCD,DISP_OFF	      ;display off - data in RAM remains intact
	CALL LCD_WRITE_INST
	RETURN
;------------------------------------------------------------------------------

LCD_DISP_ON_CRSR_OFF

	LOADREG TEMP_LCD,DISP_ON_CRSR_OFF	;display ON - CRSR off
	CALL LCD_WRITE_INST
	RETURN
;------------------------------------------------------------------------------

LCD_DISP_ON_CRSR_ON_NOBLINK
	LOADREG TEMP_LCD,DISP_ON_CRSR_ON_NOBLINK	;display ON - CRSR not blinking
	CALL LCD_WRITE_INST
	RETURN
;------------------------------------------------------------------------------

LCD_DISP_ON_CRSR_ON_BLINK
	LOADREG TEMP_LCD,DISP_ON_CRSR_ON_BLINK	;display ON - CRSR blinking
	CALL LCD_WRITE_INST
	RETURN
;------------------------------------------------------------------------------

LCD_DISP_CLR_CRSR_HOME
	LOADREG TEMP_LCD,DISP_CLR_CRSR_HOME	;display clear and CRSR home
	CALL LCD_WRITE_INST
	RETURN
;------------------------------------------------------------------------------





